@@ -27,13 +27,16 @@ NULL
 #' @param verbose logical(1). Should extra debugging messages be shown. Defaults to \code{FALSE}.
 #' @param path character. A vector path for a position within the structure of a tabletree. Each element represents a subsequent choice amongst the children of the previous choice.
 #' @param label character(1). A label (not to be confused with the name) for the object/structure.
-#' @param visible_label logical(1). Should the \emph{split} label be visible in the resulting table.
+#' @param label_pos character(1). Location the variable label should be displayed, Accepts  hidden (default for non-analyze row splits), visible, topleft, and - for analyze splits only - default.  For analyze calls, \code{default} indicates that the variable
+#' should be visible if and only if multiple variables are analyzed at the same level of nesting.
+# #' @param visible_label logical(1). Should the \emph{split} label be visible in the resulting table.
 #' @param cvar character(1). The variable, if any, which the content function should accept. Defaults to NA.
 #' @param topleft character. Override values for the "top left" material to be displayed during printing.
 #' @return NULL (this is an argument template dummy function)
 #' @rdname gen_args
 gen_args <- function(df, alt_counts_df, spl, pos, tt, tr, verbose, colwidths, obj, x,
-                     value, object, path, label, visible_label, cvar, topleft, ...) NULL
+                     value, object, path, label, label_pos, # visible_label,
+                     cvar, topleft, ...) NULL
 
 
 
@@ -71,7 +74,6 @@ gen_args <- function(df, alt_counts_df, spl, pos, tt, tr, verbose, colwidths, ob
 #' @param cutlabelfun function. Function which returns either labels for the cuts or NULL when passed the return value of \code{cutfun}
 #' @param cumulative logical. Should the cuts be treated as cumulative. Defaults to \code{FALSE}
 #' @param cutfun function. Function which accepts the \emph{full vector} of \code{var} values and returns cut points to be used (via \code{cut}) when splitting data during tabulation
-#' @param incl_all logical(1). Should an "all" comparison column be created. Defaults to \code{FALSE}.
 #' @param indent_mod numeric. Modifier for the default indent position for the structure created by this function(subtable, content table, or row) \emph{and all of that structure's children}. Defaults to 0, which corresponds to the unmodified default behavior.
 #' @param show_labels character(1). Should the variable labels for corresponding to the variable(s) in \code{vars} be visible in the resulting table.
 #' @param table_names character. Names for the tables representing each atomic analysis. Defaults to \code{var}.
@@ -83,7 +85,8 @@ lyt_args <- function( lyt, var, vars, label, labels_var, varlabels, varnames, sp
                      split_label, afun, inclNAs, valorder,
                      ref_group, compfun, label_fstr, child_labels, extra_args, name,
                      cuts, cutlabels, cutfun, cutlabelfun, cumulative,
-                     incl_all, indent_mod, show_labels, visible_label, var_labels, cvar,
+                     indent_mod, show_labels, label_pos, #visible_label,
+                     var_labels, cvar,
                      table_names, topleft) NULL
 
 
@@ -104,8 +107,17 @@ lyt_args <- function( lyt, var, vars, label, labels_var, varlabels, varnames, sp
 #' @param cindent_mod numeric(1). The indent modifier for the content tables generated by this split.
 #' @param cextra_args list. Extra arguments to be passed to the content function when tabulating row group summaries.
 #' @param child_names character. Names to be given to the sub splits contained by a compound split (typically a AnalyzeMultiVars split object).
+#' @param title character(1). Main title. Ignored for subtables.
+#' @param subtitles character. Subtitles. Ignored for subtables.
+#' @param main_footer character. Main global (non-referential) footer materials.
+#' @param prov_footer character. Provenance-related global footer materials. Generally should not be modified by hand.
+#' @param footnotes list or NULL. Referential footnotes to be applied at current level
+
 #' @rdname constr_args
-constr_args <- function(kids, cont, lev, iscontent, cinfo, labelrow, vals, cspan, visible_label, cindent_mod, cvar, label, cextra_args, child_names) NULL
+constr_args <- function(kids, cont, lev, iscontent, cinfo, labelrow, vals, cspan, label_pos, #visible_label,
+                        cindent_mod, cvar, label, cextra_args, child_names,
+                        title, subtitles, main_footer, prov_footer,
+                        footnotes) NULL
 
 #' Compatability Arg Conventions
 #' @name compat_args
