@@ -435,25 +435,28 @@ setMethod(".applysplit_partlabels", "VarLevelSplit",
     vlabelname <- spl_labelvar(spl)
     varvec = df[[varname]]
     if(is.null(vals)) {
-
         vals = if(is.factor(varvec))
                    levels(varvec)
                else
                    unique(varvec)
     }
     if(is.null(labels)) {
-        if(varname == vlabelname)
+        if(varname == vlabelname) {
             labels = vals
-        else if (is.factor(df[[vlabelname]]))
-            labels = levels(df[varvec %in% vals, ][[vlabelname]])
-        else {
+            ## } else if (is.factor(df[[vlabelname]])) {
+            ##     labels = levels(df[varvec %in% vals, ][[vlabelname]])
+        } else {
+            labfact <- is.factor(df[[vlabelname]])
+            lablevs <- if(labfact) levels(df[[vlabelname]]) else NULL
             labels = sapply(vals, function(v) {
                 vlabel = unique(df[varvec == v,
                                    vlabelname, drop = TRUE])
                 ## TODO remove this once 1-to-1 value-label map is enforced elsewhere.
                 stopifnot(length(vlabel) < 2)
                 if(length(vlabel) == 0)
-                    vlabel = ""
+                vlabel = ""
+                else if(labfact)
+                    vlabel <- lablevs[vlabel]
                 vlabel
             })
         }
@@ -615,17 +618,21 @@ drop_and_remove_levels <- function(excl) {
 #' @export
 #'
 reorder_split_levels = function(neworder, newlabels = neworder, drlevels = TRUE) {
+    if(length(neworder) != length(newlabels)) {
+        stop("Got mismatching lengths for neworder and newlabels.")
+    }
     function(df, spl,  trim, ...) {
-        df2 = df
+         df2 <- df
         valvec <- df2[[spl_payload(spl)]]
-        vals = if(is.factor(valvec)) levels(valvec) else unique(valvec)
+        vals <- if(is.factor(valvec)) levels(valvec) else unique(valvec)
         if(!drlevels)
             neworder <- c(neworder, setdiff(vals, neworder))
         df2[[spl_payload(spl)]] = factor(valvec, levels = neworder)
         if(drlevels) {
-            df2[[spl_payload(spl)]] = droplevels(df2[[spl_payload(spl)]] )
-            neworder = levels(df2[[spl_payload(spl)]])
-            newlabels = newlabels[newlabels %in% neworder]
+            orig_order <- neworder
+            df2[[spl_payload(spl)]] <- droplevels(df2[[spl_payload(spl)]] )
+            neworder <- levels(df2[[spl_payload(spl)]])
+            newlabels <- newlabels[orig_order %in% neworder]
         }
         spl_child_order(spl) <- neworder
         .apply_split_inner(spl, df2, vals = neworder, labels = newlabels, trim = trim)
@@ -634,7 +641,7 @@ reorder_split_levels = function(neworder, newlabels = neworder, drlevels = TRUE)
 
 
 #' @rdname split_funcs
-#' @param innervar character(1). Variable whose factor levels should be trimmed (ie empty levels dropped) \emph{separately within each grouping defined at this point in the structure}
+#' @param innervar character(1). Variable whose factor levels should be trimmed (e.g., empty levels dropped) \emph{separately within each grouping defined at this point in the structure}
 #' @export
 trim_levels_in_group = function(innervar) {
     myfun = function(df, spl, vals = NULL, labels = NULL, trim = FALSE) {
@@ -660,6 +667,33 @@ trim_levels_in_group = function(innervar) {
     myfun
 }
 
+#' @rdname split_funcs
+#' @param outervar character(1). Parent split variable to trim \code{innervar} levels within. Must appear in map
+#' @param map data.frame. Data frame mapping \code{outervar} values  to allowable \code{innervar} values. If no map exists a-priori, use
+#' @export
+trim_levels_by_map = function(innervar, outervar, map = NULL) {
+    if(is.null(map))
+        stop("no map dataframe was provided. Use trim_levels_in_group to trim combinations present in the data being tabulated.")
+    myfun = function(df, spl, vals = NULL, labels = NULL, trim = FALSE) {
+        ret = .apply_split_inner(spl, df, vals = vals, labels = labels, trim = trim)
+
+        outval <- unique(as.character(df[[outervar]]))
+        oldlevs <- spl_child_order(spl)
+        newlevs <- oldlevs[oldlevs %in% map[as.character(map[[outervar]]) == outval, innervar, drop =TRUE]]
+
+        keep <- ret$values %in% newlevs
+        ret <- lapply(ret, function(x) x[keep])
+        ret$datasplit <- lapply(ret$datasplit, function(df) {
+            df[[innervar]] <- factor(as.character(df[[innervar]]), levels = newlevs)
+            df
+        })
+        ret$labels <- as.character(ret$labels) # TODO
+        ret
+    }
+    myfun
+}
+
+
 .add_combo_part_info = function(part, df, valuename, levels, label, extras, first = TRUE) {
 
     ##    value = LevelComboSplitValue(levels, extras, comboname = valuename, label = label)
